@@ -26,7 +26,7 @@ This WSGI component:
 * Collects and forwards identity information based on a valid token
   such as user name, domain, project, etc.
 
-Refer to: http://docs.openstack.org/developer/keystonemiddleware/\
+Refer to: https://docs.openstack.org/keystonemiddleware/latest/\
 middlewarearchitecture.html
 
 
@@ -72,6 +72,11 @@ HTTP_X_IDENTITY_STATUS, HTTP_X_SERVICE_IDENTITY_STATUS
     presented. This allows the underlying service to determine if a
     denial should use ``401 Unauthenticated`` or ``403 Forbidden``.
 
+HTTP_OPENSTACK_SYSTEM_SCOPE
+    A string relaying system information about the token's scope. This
+    attribute is only present if the token is system-scoped. The string ``all``
+    means the token is scoped to the entire deployment system.
+
 HTTP_X_DOMAIN_ID, HTTP_X_SERVICE_DOMAIN_ID
     Identity service managed unique identifier, string. Only present if
     this is a domain-scoped token.
@@ -215,7 +220,6 @@ object is stored.
 import binascii
 import copy
 import datetime
-import logging
 
 from keystoneauth1 import access
 from keystoneauth1 import adapter
@@ -225,8 +229,10 @@ from keystoneauth1 import loading
 from keystoneauth1.loading import session as session_loading
 from keystoneclient.common import cms
 from keystoneclient import exceptions as ksc_exceptions
+import oslo_cache
+from oslo_config import cfg
+from oslo_log import log as logging
 from oslo_serialization import jsonutils
-import six
 import webob.dec
 
 from keystonemiddleware._common import config
@@ -240,11 +246,12 @@ from keystonemiddleware.auth_token import _request
 from keystonemiddleware.auth_token import _revocations
 from keystonemiddleware.auth_token import _signing_dir
 from keystonemiddleware.auth_token import _user_plugin
-from keystonemiddleware.i18n import _, _LC, _LE, _LI, _LW
+from keystonemiddleware.i18n import _
 
 
 _LOG = logging.getLogger(__name__)
 _CACHE_INVALID_INDICATOR = 'invalid'
+oslo_cache.configure(cfg.CONF)
 
 
 AUTH_TOKEN_OPTS = [
@@ -309,10 +316,15 @@ class BaseAuthProtocol(object):
     def __init__(self,
                  app,
                  log=_LOG,
-                 enforce_token_bind=_BIND_MODE.PERMISSIVE):
+                 enforce_token_bind=_BIND_MODE.PERMISSIVE,
+                 service_token_roles=None,
+                 service_token_roles_required=False):
         self.log = log
         self._app = app
         self._enforce_token_bind = enforce_token_bind
+        self._service_token_roles = set(service_token_roles or [])
+        self._service_token_roles_required = service_token_roles_required
+        self._service_token_warning_emitted = False
 
     @webob.dec.wsgify(RequestClass=_request._AuthTokenRequest)
     def __call__(self, req):
@@ -341,20 +353,7 @@ class BaseAuthProtocol(object):
         """
         user_auth_ref = None
         serv_auth_ref = None
-
-        if request.user_token:
-            self.log.debug('Authenticating user token')
-            try:
-                data, user_auth_ref = self._do_fetch_token(request.user_token)
-                self._validate_token(user_auth_ref)
-                if not request.service_token:
-                    self._confirm_token_bind(user_auth_ref, request)
-            except ksm_exceptions.InvalidToken:
-                self.log.info(_LI('Invalid user token'))
-                request.user_token_valid = False
-            else:
-                request.user_token_valid = True
-                request.token_info = data
+        allow_expired = False
 
         if request.service_token:
             self.log.debug('Authenticating service token')
@@ -363,15 +362,62 @@ class BaseAuthProtocol(object):
                 self._validate_token(serv_auth_ref)
                 self._confirm_token_bind(serv_auth_ref, request)
             except ksm_exceptions.InvalidToken:
-                self.log.info(_LI('Invalid service token'))
+                self.log.info('Invalid service token')
                 request.service_token_valid = False
             else:
-                request.service_token_valid = True
+                # FIXME(jamielennox): The new behaviour for service tokens is
+                # that they have to pass the policy check to be allowed.
+                # Previously any token was accepted here. For now we will
+                # continue to mark service tokens as valid if they are valid
+                # but we will only allow service role tokens to do
+                # allow_expired. In future we should reject any token that
+                # isn't a service token here.
+                role_names = set(serv_auth_ref.role_names)
+                check = self._service_token_roles.intersection(role_names)
+                role_check_passed = bool(check)
+
+                # if service_token_role_required then the service token is only
+                # valid if the roles check out. Otherwise at this point it is
+                # true because keystone has already validated it.
+                if self._service_token_roles_required:
+                    request.service_token_valid = role_check_passed
+                else:
+                    if not self._service_token_warning_emitted:
+                        self.log.warning('A valid token was submitted as '
+                                         'a service token, but it was not '
+                                         'a valid service token. This is '
+                                         'incorrect but backwards '
+                                         'compatible behaviour. This will '
+                                         'be removed in future releases.')
+                        # prevent log spam on every single request
+                        self._service_token_warning_emitted = True
+
+                    request.service_token_valid = True
+
+                # allow_expired always requires passing the role check.
+                allow_expired = role_check_passed
+
+        if request.user_token:
+            self.log.debug('Authenticating user token')
+            try:
+                data, user_auth_ref = self._do_fetch_token(
+                    request.user_token,
+                    allow_expired=allow_expired)
+                self._validate_token(user_auth_ref,
+                                     allow_expired=allow_expired)
+                if not request.service_token:
+                    self._confirm_token_bind(user_auth_ref, request)
+            except ksm_exceptions.InvalidToken:
+                self.log.info('Invalid user token')
+                request.user_token_valid = False
+            else:
+                request.user_token_valid = True
+                request.token_info = data
 
         request.token_auth = _user_plugin.UserAuthPlugin(user_auth_ref,
                                                          serv_auth_ref)
 
-    def _validate_token(self, auth_ref):
+    def _validate_token(self, auth_ref, allow_expired=False):
         """Perform the validation steps on the token.
 
         :param auth_ref: The token data
@@ -380,20 +426,23 @@ class BaseAuthProtocol(object):
         :raises exc.InvalidToken: if token is rejected
         """
         # 0 seconds of validity means it is invalid right now
-        if auth_ref.will_expire_soon(stale_duration=0):
+        if (not allow_expired) and auth_ref.will_expire_soon(stale_duration=0):
             raise ksm_exceptions.InvalidToken(_('Token authorization failed'))
 
-    def _do_fetch_token(self, token):
+    def _do_fetch_token(self, token, **kwargs):
         """Helper method to fetch a token and convert it into an AccessInfo."""
-        data = self.fetch_token(token)
+        # NOTE(edmondsw): strip the token to remove any whitespace that may
+        # have been passed along in the header per bug 1689468
+        token = token.strip()
+        data = self.fetch_token(token, **kwargs)
 
         try:
             return data, access.create(body=data, auth_token=token)
         except Exception:
-            self.log.warning(_LW('Invalid token contents.'), exc_info=True)
+            self.log.warning('Invalid token contents.', exc_info=True)
             raise ksm_exceptions.InvalidToken(_('Token authorization failed'))
 
-    def fetch_token(self, token):
+    def fetch_token(self, token, **kwargs):
         """Fetch the token data based on the value in the header.
 
         Retrieve the data associated with the token value that was in the
@@ -401,13 +450,17 @@ class BaseAuthProtocol(object):
         whatever is required.
 
         :param str token: The token present in the request header.
+        :param dict kwargs: Additional keyword arguments may be passed through
+                            here to support new features. If an implementation
+                            is not aware of how to use these arguments it
+                            should ignore them.
 
         :raises exc.InvalidToken: if token is invalid.
 
         :returns: The token data
         :rtype: dict
         """
-        raise NotImplemented()
+        raise NotImplementedError()
 
     def process_response(self, response):
         """Do whatever you'd like to the response.
@@ -439,7 +492,7 @@ class BaseAuthProtocol(object):
                 # no bind provided and none required
                 return
             else:
-                self.log.info(_LI('No bind information present in token.'))
+                self.log.info('No bind information present in token.')
                 self._invalid_user_token()
 
         # get the named mode if bind_mode is not one of the predefined
@@ -449,20 +502,20 @@ class BaseAuthProtocol(object):
             name = self._enforce_token_bind
 
         if name and name not in auth_ref.bind:
-            self.log.info(_LI('Named bind mode %s not in bind information'),
+            self.log.info('Named bind mode %s not in bind information',
                           name)
             self._invalid_user_token()
 
-        for bind_type, identifier in six.iteritems(auth_ref.bind):
+        for bind_type, identifier in auth_ref.bind.items():
             if bind_type == _BIND_MODE.KERBEROS:
                 if req.auth_type != 'negotiate':
-                    self.log.info(_LI('Kerberos credentials required and '
-                                      'not present.'))
+                    self.log.info('Kerberos credentials required and '
+                                  'not present.')
                     self._invalid_user_token()
 
                 if req.remote_user != identifier:
-                    self.log.info(_LI('Kerberos credentials do not match '
-                                      'those in bind.'))
+                    self.log.info('Kerberos credentials do not match '
+                                  'those in bind.')
                     self._invalid_user_token()
 
                 self.log.debug('Kerberos bind authentication successful.')
@@ -475,8 +528,8 @@ class BaseAuthProtocol(object):
 
             else:
                 self.log.info(
-                    _LI('Couldn`t verify unknown bind: %(bind_type)s: '
-                        '%(identifier)s.'),
+                    'Couldn`t verify unknown bind: %(bind_type)s: '
+                    '%(identifier)s.',
                     {'bind_type': bind_type, 'identifier': identifier})
                 self._invalid_user_token()
 
@@ -489,17 +542,29 @@ class AuthProtocol(BaseAuthProtocol):
 
     def __init__(self, app, conf):
         log = logging.getLogger(conf.get('log_name', __name__))
-        log.info(_LI('Starting Keystone auth_token middleware'))
+        log.info('Starting Keystone auth_token middleware')
 
         self._conf = config.Config('auth_token',
                                    _base.AUTHTOKEN_GROUP,
                                    list_opts(),
                                    conf)
+        if self._conf.oslo_conf_obj != cfg.CONF:
+            oslo_cache.configure(self._conf.oslo_conf_obj)
+
+        token_roles_required = self._conf.get('service_token_roles_required')
+
+        if not token_roles_required:
+            log.warning('AuthToken middleware is set with '
+                        'keystone_authtoken.service_token_roles_required '
+                        'set to False. This is backwards compatible but '
+                        'deprecated behaviour. Please set this to True.')
 
         super(AuthProtocol, self).__init__(
             app,
             log=log,
-            enforce_token_bind=self._conf.get('enforce_token_bind'))
+            enforce_token_bind=self._conf.get('enforce_token_bind'),
+            service_token_roles=self._conf.get('service_token_roles'),
+            service_token_roles_required=token_roles_required)
 
         # delay_auth_decision means we still allow unauthenticated requests
         # through and we let the downstream service make the final decision
@@ -508,19 +573,24 @@ class AuthProtocol(BaseAuthProtocol):
             'include_service_catalog')
         self._hash_algorithms = self._conf.get('hash_algorithms')
 
+        self._auth = self._create_auth_plugin()
+        self._session = self._create_session()
         self._identity_server = self._create_identity_server()
 
-        self._auth_uri = self._conf.get('auth_uri')
-        if not self._auth_uri:
+        self._www_authenticate_uri = self._conf.get('www_authenticate_uri')
+        if not self._www_authenticate_uri:
+            self._www_authenticate_uri = self._conf.get('auth_uri')
+        if not self._www_authenticate_uri:
             self.log.warning(
-                _LW('Configuring auth_uri to point to the public identity '
-                    'endpoint is required; clients may not be able to '
-                    'authenticate against an admin endpoint'))
+                'Configuring www_authenticate_uri to point to the public '
+                'identity endpoint is required; clients may not be able to '
+                'authenticate against an admin endpoint')
 
             # FIXME(dolph): drop support for this fallback behavior as
             # documented in bug 1207517.
 
-            self._auth_uri = self._identity_server.auth_uri
+            self._www_authenticate_uri = \
+                self._identity_server.www_authenticate_uri
 
         self._signing_directory = _signing_dir.SigningDirectory(
             directory_name=self._conf.get('signing_dir'), log=self.log)
@@ -567,10 +637,11 @@ class AuthProtocol(BaseAuthProtocol):
 
         if not (user_status and service_status == 'Confirmed'):
             if self._delay_auth_decision:
-                self.log.info(_LI('Deferring reject downstream'))
+                self.log.debug('Deferring reject downstream')
             else:
-                self.log.info(_LI('Rejecting request'))
-                message = 'The request you have made requires authentication.'
+                self.log.info('Rejecting request')
+                message = _('The request you have made requires '
+                            'authentication.')
                 body = {'error': {
                     'code': 401,
                     'title': 'Unauthorized',
@@ -579,6 +650,7 @@ class AuthProtocol(BaseAuthProtocol):
                 raise webob.exc.HTTPUnauthorized(
                     body=jsonutils.dumps(body),
                     headers=self._reject_auth_headers,
+                    charset='UTF-8',
                     content_type='application/json')
 
         if request.user_token_valid:
@@ -587,6 +659,10 @@ class AuthProtocol(BaseAuthProtocol):
             if self._include_service_catalog:
                 request.set_service_catalog_headers(request.token_auth.user)
 
+        if request.token_auth:
+            request.token_auth._auth = self._auth
+            request.token_auth._session = self._session
+
         if request.service_token and request.service_token_valid:
             request.set_service_headers(request.token_auth.service)
 
@@ -608,7 +684,7 @@ class AuthProtocol(BaseAuthProtocol):
 
     @property
     def _reject_auth_headers(self):
-        header_val = 'Keystone uri=\'%s\'' % self._auth_uri
+        header_val = 'Keystone uri="%s"' % self._www_authenticate_uri
         return [('WWW-Authenticate', header_val)]
 
     def _token_hashes(self, token):
@@ -645,7 +721,7 @@ class AuthProtocol(BaseAuthProtocol):
             if cached:
                 return cached
 
-    def fetch_token(self, token):
+    def fetch_token(self, token, allow_expired=False):
         """Retrieve a token from either a PKI bundle or the identity server.
 
         :param str token: token id
@@ -680,7 +756,9 @@ class AuthProtocol(BaseAuthProtocol):
             else:
                 data = self._validate_offline(token, token_hashes)
                 if not data:
-                    data = self._identity_server.verify_token(token)
+                    data = self._identity_server.verify_token(
+                        token,
+                        allow_expired=allow_expired)
 
                 self._token_cache.set(token_hashes[0], data)
 
@@ -688,14 +766,15 @@ class AuthProtocol(BaseAuthProtocol):
                 ksa_exceptions.RequestTimeout,
                 ksm_exceptions.RevocationListError,
                 ksm_exceptions.ServiceError) as e:
-            self.log.critical(_LC('Unable to validate token: %s'), e)
-            raise webob.exc.HTTPServiceUnavailable()
+            self.log.critical('Unable to validate token: %s', e)
+            raise webob.exc.HTTPServiceUnavailable(
+                'The Keystone service is temporarily unavailable.')
         except ksm_exceptions.InvalidToken:
             self.log.debug('Token validation failure.', exc_info=True)
             if token_hashes:
                 self._token_cache.set(token_hashes[0],
                                       _CACHE_INVALID_INDICATOR)
-            self.log.warning(_LW('Authorization failed for token'))
+            self.log.warning('Authorization failed for token')
             raise
 
         return data
@@ -715,12 +794,19 @@ class AuthProtocol(BaseAuthProtocol):
             self._revocations.check(token_hashes)
             verified = self._cms_verify(token_data, inform)
         except ksc_exceptions.CertificateConfigError:
-            self.log.warning(_LW('Fetch certificate config failed, '
-                                 'fallback to online validation.'))
+            self.log.warning('Fetch certificate config failed, '
+                             'fallback to online validation.')
         except ksm_exceptions.RevocationListError:
-            self.log.warning(_LW('Fetch revocation list failed, '
-                                 'fallback to online validation.'))
+            self.log.warning('Fetch revocation list failed, '
+                             'fallback to online validation.')
         else:
+            self.log.warning('auth_token middleware received a PKI/Z token. '
+                             'This form of token is deprecated and has been '
+                             'removed from keystone server and will be '
+                             'removed from auth_token middleware in the Rocky '
+                             'release. Please contact your administrator '
+                             'about upgrading keystone and the token format.')
+
             data = jsonutils.loads(verified)
 
             audit_ids = None
@@ -736,8 +822,8 @@ class AuthProtocol(BaseAuthProtocol):
 
             return data
 
-    def _validate_token(self, auth_ref):
-        super(AuthProtocol, self)._validate_token(auth_ref)
+    def _validate_token(self, auth_ref, **kwargs):
+        super(AuthProtocol, self)._validate_token(auth_ref, **kwargs)
 
         if auth_ref.version == 'v2.0' and not auth_ref.project_id:
             msg = _('Unable to determine service tenancy.')
@@ -760,7 +846,7 @@ class AuthProtocol(BaseAuthProtocol):
                                       inform=inform).decode('utf-8')
             except (ksc_exceptions.CMSError,
                     cms.subprocess.CalledProcessError) as err:
-                self.log.warning(_LW('Verify error: %s'), err)
+                self.log.warning('Verify error: %s', err)
                 msg = _('Token authorization failed')
                 raise ksm_exceptions.InvalidToken(msg)
 
@@ -777,7 +863,7 @@ class AuthProtocol(BaseAuthProtocol):
             except ksc_exceptions.CertificateConfigError as err:
                 # if this is still occurring, something else is wrong and we
                 # need err.output to identify the problem
-                self.log.error(_LE('CMS Verify output: %s'), err.output)
+                self.log.error('CMS Verify output: %s', err.output)
                 raise
 
     def _fetch_signing_cert(self):
@@ -790,7 +876,7 @@ class AuthProtocol(BaseAuthProtocol):
             self._SIGNING_CA_FILE_NAME,
             self._identity_server.fetch_ca_cert())
 
-    def _get_auth_plugin(self):
+    def _create_auth_plugin(self):
         # NOTE(jamielennox): Ideally this would use load_from_conf_options
         # however that is not possible because we have to support the override
         # pattern we use in _conf.get. This function therefore does a manual
@@ -829,25 +915,24 @@ class AuthProtocol(BaseAuthProtocol):
         getter = lambda opt: self._conf.get(opt.dest, group=group)
         return plugin_loader.load_from_options_getter(getter)
 
-    def _create_identity_server(self):
+    def _create_session(self, **kwargs):
         # NOTE(jamielennox): Loading Session here should be exactly the
         # same as calling Session.load_from_conf_options(CONF, GROUP)
         # however we can't do that because we have to use _conf.get to
         # support the paste.ini options.
-        sess = session_loading.Session().load_from_options(
-            cert=self._conf.get('certfile'),
-            key=self._conf.get('keyfile'),
-            cacert=self._conf.get('cafile'),
-            insecure=self._conf.get('insecure'),
-            timeout=self._conf.get('http_connect_timeout'),
-            user_agent=self._conf.user_agent,
-        )
+        kwargs.setdefault('cert', self._conf.get('certfile'))
+        kwargs.setdefault('key', self._conf.get('keyfile'))
+        kwargs.setdefault('cacert', self._conf.get('cafile'))
+        kwargs.setdefault('insecure', self._conf.get('insecure'))
+        kwargs.setdefault('timeout', self._conf.get('http_connect_timeout'))
+        kwargs.setdefault('user_agent', self._conf.user_agent)
 
-        auth_plugin = self._get_auth_plugin()
+        return session_loading.Session().load_from_options(**kwargs)
 
+    def _create_identity_server(self):
         adap = adapter.Adapter(
-            sess,
-            auth=auth_plugin,
+            self._session,
+            auth=self._auth,
             service_type='identity',
             interface='admin',
             region_name=self._conf.get('region_name'),
@@ -862,7 +947,14 @@ class AuthProtocol(BaseAuthProtocol):
             include_service_catalog=self._include_service_catalog,
             requested_auth_version=auth_version)
 
+    def _create_oslo_cache(self):
+        # having this as a function makes test mocking easier
+        region = oslo_cache.create_region()
+        oslo_cache.configure_cache_region(self._conf.oslo_conf_obj, region)
+        return region
+
     def _token_cache_factory(self):
+
         security_strategy = self._conf.get('memcache_security_strategy')
 
         cache_kwargs = dict(
